package com.innoq.procurement.resources;

import com.innoq.procurement.entities.Customer;
import static com.innoq.procurement.Utilities.*;
import com.innoq.procurement.Utilities;

import javax.ws.rs.*;
import javax.ws.rs.core.Response;
import java.util.List;
import java.net.URI;

import nu.xom.*;


@Path("customers/")
public class CustomersResource {

    @GET @ProduceMime("text/plain")
    public String getAsPlainText() {
        return toString() +  "\n\n";
    }

    @GET @ProduceMime("application/vnd.innoq.customers+xml")
    public String getAsXml() {
        List<Customer> customers = Customer.findAll();
        Element root = new Element("customers", Utilities.NAMESPACE);
        for (Customer customer : customers) {
            Element customerElement = new Element("customer", Utilities.NAMESPACE);
            customerElement.appendChild(customer.getName());
            root.appendChild(customerElement);
        }
        return elementToXmlString(root);
    }

    @POST @ConsumeMime("application/vnd.innoq.customer+xml")
    public Response newCustomer(String body) {
        Builder b = new Builder();
        try {
            System.out.println("Received: " + body);
            Document doc = b.build(body, ".");
            Customer c = new Customer(doc.query("/i:customer/i:name", 
                    new XPathContext("i", Utilities.NAMESPACE)).get(0).getValue());
            int id = Customer.add(c);
            URI uri = new URI("./customers/" + id);
            System.out.println("Returning: " + uri.toString());
            return Response.created(uri).build();

        } catch (Exception e) {
            e.printStackTrace();
            return Response.status(400).entity("Please send well-formed XML\n").type("text/plain").build();
        }

    }

    @Path("{id}")
    public CustomerResource customerById(@PathParam("id") int id) {
        return new CustomerResource(Customer.get(id));
    }




}
