package com.innoq.procurement;

import com.innoq.procurement.entities.Customer;
import static com.innoq.procurement.Utilities.NAMESPACE;
import static com.innoq.procurement.Utilities.writeElementToStream;

import javax.ws.rs.ext.MessageBodyWriter;
import javax.ws.rs.ext.MessageBodyReader;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import java.io.OutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.util.List;

import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.XPathContext;
import nu.xom.Element;

public class CustomRW implements MessageBodyWriter, MessageBodyReader  {
    public boolean isWriteable(Class type) {
        return Customer.class.isAssignableFrom(type)
                || List.class.isAssignableFrom(type);
    }

    public boolean isReadable(Class type) {
        return Customer.class.isAssignableFrom(type);
    }

    public long getSize(Object o) {
        return -1;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public void writeTo(Object o, MediaType mediaType, MultivaluedMap multivaluedMap, OutputStream outputStream) throws IOException {
        if (o instanceof Customer) {
            Customer customer = (Customer)o;
            if (mediaType.getType().compareTo("application") == 0 &&
                    mediaType.getSubtype().compareTo("vnd.innoq.customer+xml") == 0) {
                Element root = new Element("customer", NAMESPACE);
                Element name = new Element("name", NAMESPACE);
                name.appendChild(customer.getName());
                root.appendChild(name);
                writeElementToStream(root, outputStream);
            } else if (mediaType.getType().compareTo("text") == 0 &&
                    mediaType.getSubtype().compareTo("plain") == 0) {
                OutputStreamWriter osw = new OutputStreamWriter(outputStream);
                osw.write("Customer: \"");
                osw.write(customer.getName());
                osw.write("\"\n");
                osw.close();
            }
        } else if (o instanceof List) {
            List<Customer> customers = (List<Customer>) o;
            if (mediaType.getType().compareTo("application") == 0 &&
                    mediaType.getSubtype().compareTo("vnd.innoq.customer+xml") == 0) {
                Element root = new Element("customers", NAMESPACE);
                for (Customer customer : customers) {
                    if (customer != null) {
                        Element customerElement = new Element("customer", NAMESPACE);
                        customerElement.appendChild(customer.getName());
                        root.appendChild(customerElement);
                    }
                }
                writeElementToStream(root, outputStream);
            } else if (mediaType.getType().compareTo("text") == 0 &&
                    mediaType.getSubtype().compareTo("plain") == 0) {
                OutputStreamWriter osw = new OutputStreamWriter(outputStream);
                osw.write("Customer list: \"");
                for (Customer customer : customers) {
                    osw.write(customer.getName());
                    osw.write("\"\n");
                    osw.close();
                }
            }
        }
    }

    public Object readFrom(Class aClass, MediaType mediaType, MultivaluedMap multivaluedMap, InputStream inputStream) throws IOException {
        Customer customer = null;
        Builder b = new Builder();
        Document doc = null;
        try {
            doc = b.build(inputStream);
            String name = doc.query("/i:customer/i:name", new XPathContext("i", Utilities.NAMESPACE)).get(0).getValue();
            customer = new Customer(name);
        } catch (Exception e) {
            IOException ioex = new IOException(e.getMessage());
            ioex.initCause(e);
            throw ioex;
        }
        return customer;
    }
}
